''Driver for the PropBot


CON
        _clkmode = xtal1 + pll16x                                               'Standard clock mode * crystal frequency = 80 MHz
        _xinfreq = 5_000_000

        ' Pins for ADC operations
        Vclk_p        = 8     ' clock pin
        Vn_p          = 9     ' data in
        Vo_p          = 9     ' data out
        Vcs_p         = 10    ' CS pin

        ' Set pins and Baud rate for XBee comms  
        XB_Rx     = 23              ' XBee Dout
        XB_Tx     = 22              ' XBee Din
        XB_Baud   = 9600

        ' Carriage return value
        CR = 13

        ' Newline value
        NL = 10

        ' Timeout value for XBEE operations
        timeout = 500

        ' Pins for PWM
        ' Left motor or Motor 1
        left_enablePin = 16
        left_forwardPin = 17
        left_reversePin = 18
        ' Right motor or Motor 2
        right_enablePin = 19
        right_forwardPin = 20
        right_reversePin = 21

VAR
  
   
OBJ
  ADC           : "ADC_INPUT_DRIVER"
  compass       : "HMC5883L"
  f             : "FloatMath"
  fs            : "FloatString"
  pwmL          : "PWMMotorDriver"
  pwmR          : "PWMMotorDriver"
  servo         : "Servo4"
  sonar         : "gp2d12"
  XB            : "XBee_Object_2" 
  
  
PUB main_loop | DataIn, voltage, ld, rd, pan, tilt
  compass.Start(28,29)
  servo.start(2300,6,1650,7,0,0,0,0)
  pwmL.start(left_enablePin,left_forwardPin,left_reversePin,40000)
  pwmR.start(right_enablePin,right_forwardPin,right_reversePin,40000)
  ADC.start(Vo_p, Vn_p, Vclk_p, Vcs_p, 4, 4, 12, 1, false)
  XB.Delay(2000)
  XB.Start(XB_Rx, XB_Tx, 0, XB_Baud)
  XB.AT_Init
  XB.RxFlush
  REPEAT
    DataIn := XB.Rx
    Case DataIn
      "S","s":
        XB.RxFlush
        XB.AT_Config(string("ATDB"))
        DataIn := XB.RxHexTime(500)
        If DataIn <> -1
          XB.str(string("RSSI:"))
          XB.Dec(-DataIn)
          XB.tx(NL)    'Send a newline for the ReadLine() method on the Windows controller 
      "V","v":
        voltage := convert_adc_output_code(ADC.average(1,5000))
        XB.str(string("VOLTAGE:"))
        XB.str(fs.FloatToString(voltage))
        XB.tx(NL)
      "L","l":
        ld := XB.rxDecTime(timeout)
        pwmL.SetDuty(ld)
        'XB.dec(ld)
      "R","r":
        rd := XB.rxDecTime(timeout)
        pwmR.SetDuty(rd)
        'XB.dec(rd)
      "P","p":
        pan := XB.rxDecTime(timeout)
        servo.move_to(1, pan, 10)
        'servo.wait(7)
        'XB.dec(pan)
        'XB.tx(NL)
      "T","t":
        tilt := XB.rxDecTime(timeout)
        servo.move_to(0, tilt, 10)
      "C","c":
        XB.str(string("COMPASS:"))       
        XB.Dec(compass.get_azimuth)
        XB.tx(NL)
      "D","d":
        XB.str(string("DISTANCE:"))
        XB.Dec(sonar.GetRange(ADC.Average(0, 5000)))
        XB.tx(NL)
        
    
                                                              
PUB convert_adc_output_code(adc_val) | temp
  {{Convert digital output code from MCP3204 to a voltage}}
  'temp := (((adc_val * 4800) / 4096))
  temp := f.FDiv(adc_val, 4096)
  temp := f.FMul(temp, f.FFloat(5))
  temp := f.FMul(temp, 2.33)
  RETURN temp

DAT
       
        